@@ -1198,7 +1198,16 @@ static void
 gth_image_viewer_page_real_fullscreen (GthViewerPage *base,
 				       gboolean       active)
 {
-	/* void */
+	GthImageViewerPage *self;
+	GthImageViewerTool *tool;
+
+	self = (GthImageViewerPage *) base;
+	tool = gth_image_viewer_get_tool (GTH_IMAGE_VIEWER (self->priv->viewer));
+
+	if (! GTH_IS_IMAGE_DRAGGER (tool))
+		return;
+
+	g_object_set (tool, "show-frame", ! active, NULL);
 }
 
 
@@ -1314,6 +1323,7 @@ save_image_task_completed_cb (GthTask *task,
 	}
 
 	save_data_free (data);
+	_g_object_unref (task);
 }
 
 
@@ -1375,8 +1385,6 @@ _gth_image_viewer_page_real_save (GthViewerPage *base,
 			  G_CALLBACK (save_image_task_completed_cb),
 			  data);
 	gth_browser_exec_task (GTH_BROWSER (self->priv->browser), task, GTH_TASK_FLAGS_DEFAULT);
-
-	_g_object_unref (task);
 }
 
 
