@@ -75,6 +75,8 @@ gboolean           gth_image_viewer_page_get_original_finish	(GthImageViewerPage
 								 GError			**error);
 GthTask *	   gth_original_image_task_new			(GthImageViewerPage	 *self);
 cairo_surface_t *  gth_original_image_task_get_image		(GthTask		 *task);
+void		   gth_image_viewer_page_apply_icc_profile	(GthImageViewerPage	 *self,
+								 gboolean                 apply);
 
 G_END_DECLS
 
