@@ -177,7 +177,7 @@ import_dialog_response_cb (GtkDialog *dialog,
 							    FALSE,
 							    FALSE,
 							    FALSE);
-				gth_browser_exec_task (data->browser, task, FALSE);
+				gth_browser_exec_task (data->browser, task, GTH_TASK_FLAGS_DEFAULT);
 				gtk_widget_destroy (data->dialog);
 
 				g_object_unref (task);
@@ -649,7 +649,7 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 			  data);
 
 	data->progress_dialog = gth_progress_dialog_new (GTK_WINDOW (data->browser));
-	gth_progress_dialog_add_task (GTH_PROGRESS_DIALOG (data->progress_dialog), GTH_TASK (data->service));
+	gth_progress_dialog_add_task (GTH_PROGRESS_DIALOG (data->progress_dialog), GTH_TASK (data->service), GTH_TASK_FLAGS_DEFAULT);
 
 	web_service_autoconnect (WEB_SERVICE (data->service));
 }
