@@ -198,8 +198,9 @@ void
 gth_selections_manager_update_file_info (GFile     *file,
 					 GFileInfo *info)
 {
-	int   n_selection;
-	char *name;
+	int    n_selection;
+	GIcon *icon;
+	char  *name;
 
 	n_selection = _g_file_get_n_selection (file);
 
@@ -216,21 +217,9 @@ gth_selections_manager_update_file_info (GFile     *file,
 
 	/* icon */
 
-	if (n_selection > 0) {
-		GIcon *icon;
-
-		name = g_strdup_printf ("selection%d", n_selection);
-		icon = g_themed_icon_new (name);
-		g_file_info_set_icon (info, icon);
-
-		g_object_unref (icon);
-		g_free (name);
-	}
-	else {
-		GIcon *icon = g_themed_icon_new ("selection");
-		g_file_info_set_icon (info, icon);
-		g_object_unref (icon);
-	}
+	icon = g_themed_icon_new (gth_selection_get_symbolic_icon_name (n_selection));
+	g_file_info_set_symbolic_icon (info, icon);
+	g_object_unref (icon);
 
 	/* display name */
 
@@ -391,14 +380,14 @@ gth_selections_manager_add_files (GFile *folder,
 	else
 		self->priv->files[n_selection - 1] = g_list_concat (self->priv->files[n_selection - 1], new_list);
 
+	g_mutex_unlock (&self->priv->mutex);
+
 	gth_monitor_emblems_changed (gth_main_get_default_monitor (), file_list);
 	gth_monitor_folder_changed (gth_main_get_default_monitor (),
 				    folder,
 				    file_list,
 				    GTH_MONITOR_EVENT_CREATED);
 
-	g_mutex_unlock (&self->priv->mutex);
-
 	return TRUE;
 }
 
@@ -442,13 +431,13 @@ gth_selections_manager_remove_files (GFile *folder,
 	_g_object_list_unref (self->priv->files[n_selection - 1]);
 	self->priv->files[n_selection - 1] = new_list;
 
+	g_mutex_unlock (&self->priv->mutex);
+
 	gth_monitor_folder_changed (gth_main_get_default_monitor (),
 				    folder,
 				    file_list,
 				    GTH_MONITOR_EVENT_REMOVED);
 	gth_monitor_emblems_changed (gth_main_get_default_monitor (), file_list);
-
-	g_mutex_unlock (&self->priv->mutex);
 }
 
 
@@ -536,6 +525,26 @@ gth_selections_manager_file_exists (int    n_selection,
 }
 
 
+gboolean
+gth_selections_manager_get_is_empty (int n_selection)
+{
+	GthSelectionsManager *self;
+	guint                 size;
+
+	if ((n_selection <= 0) || (n_selection > GTH_SELECTIONS_MANAGER_N_SELECTIONS))
+		return TRUE;
+
+	self = gth_selections_manager_get_default ();
+	g_mutex_lock (&self->priv->mutex);
+
+	size = g_hash_table_size (self->priv->files_hash[n_selection - 1]);
+
+	g_mutex_unlock (&self->priv->mutex);
+
+	return size == 0;
+}
+
+
 int
 _g_file_get_n_selection (GFile *file)
 {
@@ -557,3 +566,35 @@ _g_file_get_n_selection (GFile *file)
 
 	return n;
 }
+
+
+static const char * selection_icons[] = {
+	"emblem-flag-gray",
+	"emblem-flag-green",
+	"emblem-flag-red",
+	"emblem-flag-blue"
+};
+
+
+const char *
+gth_selection_get_icon_name (int n_selection)
+{
+	g_return_val_if_fail (n_selection >= 0 && n_selection <= GTH_SELECTIONS_MANAGER_N_SELECTIONS, NULL);
+	return selection_icons[n_selection];
+}
+
+
+static const char * selection_symbolic_icons[] = {
+	"emblem-flag-symbolic",
+	"emblem-flag-symbolic",
+	"emblem-flag-symbolic",
+	"emblem-flag-symbolic"
+};
+
+
+const char *
+gth_selection_get_symbolic_icon_name (int n_selection)
+{
+	g_return_val_if_fail (n_selection >= 0 && n_selection <= GTH_SELECTIONS_MANAGER_N_SELECTIONS, NULL);
+	return selection_symbolic_icons[n_selection];
+}
