@@ -39,20 +39,6 @@
 #include "main.h"
 
 
-void
-toggle_action_activated (GSimpleAction *action,
-			 GVariant      *parameter,
-			 gpointer       data)
-{
-	GVariant *state;
-
-	state = g_action_get_state (G_ACTION (action));
-	g_action_change_state (G_ACTION (action), g_variant_new_boolean (! g_variant_get_boolean (state)));
-
-	g_variant_unref (state);
-}
-
-
 static GtkWidget *
 _gth_application_get_current_window (GApplication *application)
 {
@@ -530,6 +516,7 @@ gth_browser_activate_show_previous_image (GSimpleAction *action,
 	gth_browser_show_prev_image (browser, FALSE, FALSE);
 }
 
+
 void
 gth_browser_activate_show_next_image (GSimpleAction *action,
 				      GVariant      *state,
@@ -538,3 +525,13 @@ gth_browser_activate_show_next_image (GSimpleAction *action,
 	GthBrowser *browser = GTH_BROWSER (user_data);
 	gth_browser_show_next_image (browser, FALSE, FALSE);
 }
+
+
+void
+gth_browser_activate_apply_editor_changes (GSimpleAction *action,
+					   GVariant      *state,
+					   gpointer       user_data)
+{
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	gth_browser_apply_editor_changes (browser);
+}
